<?php
/*
Plugin Name: Indizar
Plugin URI: http://www.sebaxtian.com/acerca-de/indizar
Description: A plugin to index a post using chapters.
Version: 0.5
Author: Juan Sebastián Echeverry
Author URI: http://www.sebaxtian.com
*/

/*  Copyright 2008-2009  Juan Sebastián Echeverry  (email : sebaxtian@gawab.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

add_action('init', 'ind_addbuttons');
add_action('wp_head', 'ind_header');
add_action('init', 'ind_textdomain');
add_filter('query_vars', 'ind_query_var');
add_filter('the_content', 'ind_content');
add_filter('generate_rewrite_rules','ind_rewriteRules');

define('FIRST_CHAPTER', 1);

function ind_textdomain() {
	load_plugin_textdomain('indizar', 'wp-content/plugins/indizar/lang');
}

function ind_rewriteRules($rules){
	global $wp_rewrite;
 
	// add rewrite tokens
	$keytag = '%chapter%';
	$wp_rewrite->add_rewrite_tag($keytag, '([0-9]+)', 'chapter=');

  	// rule for 'posts'
  	$post_structure = $wp_rewrite->permalink_structure . "/chapter/$keytag";
  	$post_structure = str_replace('//', '/', $post_structure);
  	$post_rewrite = $wp_rewrite->generate_rewrite_rules($post_structure, EP_PERMALINK);

	// rule for 'pages'
	/*$page_structure = $wp_rewrite->get_page_permastruct() . "/chapter/$keytag";
	$page_structure = str_replace('//', '/', $page_structure);
	$page_rewrite = $wp_rewrite->generate_rewrite_rules($page_structure);*/

	$wp_rewrite->rules = array_merge($post_rewrite, $wp_rewrite->rules);
  
	return $wp_rewrite->rules;
}

function ind_header( )
{
  	echo "<link rel='stylesheet' href='".get_bloginfo('wpurl')."/wp-content/plugins/indizar/indizar.css' type='text/css' media='screen' />";
}

function ind_query_var($vars) {
	array_push($vars, 'chapter');
	return $vars;
}

function ind_prepost($title, $content) {
	global $chapters, $page, $chapter, $first_chapter, $indizar_config;
	
	//search for configuration and poblate the global variabnle if the
	//post declared it
	$search = "@(?:<p>)*\s*\[indizar\s*:\s*(.+)\]\s*(?:</p>)*@i";
	if  (preg_match_all($search, $content, $matches))
	{
		if (is_array($matches))
		{
			$indizar_config=array();
			foreach ($matches[1] as $key =>$v0)
			{
				$search = $matches[0][$key];
				$aux = explode(',', $v0);
				$indizar_config['config'] = $aux[0];
				$indizar_config['box_size'] = $aux[1];
				$indizar_config['box_float'] = $aux[2];
				if(count($aux)>3) {
					$indizar_config['first_chapter'] = 0;
					$first_chapter = 0;
					$indizar_config['preface_title'] = $aux[3];
				} else {
					$indizar_config['first_chapter'] = 1;
					$first_chapter = 1;
				}
				$content= str_replace ($search, "", $content);
			}
		}
	}
	
	$chapters=false;
	
	$chapters[$first_chapter]=$title;
	$page_index=$first_chapter;
	
	//search for firstchapter and define it in the chapters list
	$search = "@(?:<p>)*\s*\[firstchapter\s*:\s*(.+)\]\s*(?:</p>)*@i";
	if  (preg_match_all($search, $content, $matches))
	{
		if (is_array($matches))
		{
			foreach ($matches[1] as $key =>$v0)
			{
				$search = $matches[0][$key];
				//Set the title and put a box place definition to use if we need it
				$replace= "<div class='indizar'>$v0</div><!--indizar_box-->";
				$content= str_replace ($search, $replace, $content);
				$chapters[$first_chapter]=$v0;
			}
		}
	}
	
	//search for chapter and poblate the chapters list
	$search = "@(?:<p>)*\s*\[chapter\s*:\s*(.+)\]\s*(?:</p>)*@i";
	if  (preg_match_all($search, $content, $matches))
	{
		if (is_array($matches))
		{
			foreach ($matches[1] as $key =>$v0)
			{
				$search = $matches[0][$key];
				//Set the title, put a box place definition to use if we need it
				//and create a 'chapter mark' to use as a page break
				$replace= "<!--indizar_chapter--><div class='indizar'>$v0</div><!--indizar_box-->";
				$content= str_replace ($search, $replace, $content);
				$page_index++;
				$chapters[$page_index]=$v0;
			}
		}
	}
	
	if(count($chapters)==1) 
		$chapters=false;
	
	return $content;

}

function ind_content($content)
{
	global $wp_rewrite, $wp_query, $chapter, $chapters, $post, $page, $more, $multipage, $first_chapter, $indizar_config;
	
	$first_chapter = FIRST_CHAPTER;
	$indizar_config = array();
	 
	if(is_numeric(get_post_meta($post->ID, 'indizar_first_chapter', true)))
		$first_chapter = get_post_meta($post->ID, 'indizar_first_chapter', true);

	if ($wp_rewrite->using_permalinks()) { // WordPress using Pretty Permalink structure 
		$chapter = $wp_query->query_vars['chapter'];
	} else { // WordPress using default pwrmalink structure like www.site.com/wordpress/?p=123
		$chapter = $_GET['chapter']; // now we can get variables from $_GET variable
	}
	
		
   	//search for chapters and configuration
   	$index=false;
   	$content = ind_prepost($post->post_title, $content);
	
	//If there is a configuration tag, use it to define the first chapter
	if(count($indizar_config)>0) {
		$first_chapter = $indizar_config['first_chapter'];
	}
	
	//If the URL didn't declare a chapter it' has to be the first chapter
	if(!$chapter) $chapter = $first_chapter;
	 
	// Use the entire post if we are using wp_print to print
	if(function_exists('wp_print') && ( intval(get_query_var('print')) == 1 || intval(get_query_var('printpage')) == 1 )) {
		for($page = 0; $page < $numpages; $page++) {
			$content .= $pages[$page];
		}
		
		$content = "<link rel='stylesheet' href='".get_bloginfo('wpurl')."/wp-content/plugins/indizar/indizar.css' type='text/css' media='screen' />
		".$content;
		
		$search = "@(?:<p>)*\s*\[chapters\]\s*(?:</p>)*@i";
		if  (preg_match_all($search, $content, $matches)) {
			if (is_array($matches)) {
				foreach ($matches[1] as $key =>$v0) {
					$search = $matches[0][$key];
					$content = str_replace ($search, "", $content);
				}
			}
		}
		
		$search = "@(?:<p>)*\s*\[chapters\s*:\s*(\d+)(||,left|,right)\]\s*(?:</p>)*@i";
		if  (preg_match_all($search, $content, $matches)) {
			if (is_array($matches)) {
				foreach ($matches[1] as $key =>$v0) {
					$search = $matches[0][$key];
					$content = str_replace ($search, "", $content);
				}
			}
		}
	} else {
	
		//Maybe we need a box at the begin
		$content = "<!--indizar_box_begin-->" . $content;
		
		//get the list of chapters
		$list = chap_list();
		
		//Create html code for the chapter index
		if($list && $more) {
			$index = index_list($list);
		}
		
		//Create H\htmlcode for the numbered list
		if($list && $more) {
			$numbered_list = numbered_list($list);
		}
		
		//Chapters Box without style
		$search = "@(?:<p>)*\s*\[chapters\]\s*(?:</p>)*@i";
		if  (preg_match_all($search, $content, $matches))
		{
			if (is_array($matches)) 
			{
				$search = $matches[0][0];
				$replace="";
				if($index) {
						$replace = "<div class='chapters'>".$index."</div>";
				}
				$content = str_replace ($search, $replace, $content);
			}
		}
		
		//Chapters box with style
		$search = "@(?:<p>)*\s*\[chapters\s*:\s*(\d+)(||,left|,right)\]\s*(?:</p>)*@i";
		if  (preg_match_all($search, $content, $matches))
		{
			if (is_array($matches)) 
			{

				foreach ($matches[1] as $key =>$v0)
				{
					$v1=$matches[2][$key];
					if($v1) $v1=substr($v1,1);
					$search = $matches[0][$key];
					
					$replace="";
					if($index) {
						$replace = "<div class='chapters' id='".$v1."' style='width: ".$v0."px;";
						if($v1)
							$replace.=" float: $v1;";
						$replace.="'>".$index."</div>";
					}
					$content = str_replace ($search, $replace, $content);
				}
			}
		}
		
		if ( strpos( $content, '<!--indizar_chapter-->' ) ) {
			$content = str_replace("\n<!--indizar_chapter-->", '<!--indizar_chapter-->', $content);
			$chapters_array = explode('<!--indizar_chapter-->', $content);
			
			$content = $chapters_array[$chapter-$first_chapter];
		}   
		
		//If there is a chapter list and we are showing the hole document, 
		//format document
		if($list && $more) {
			$search_box = "<!--indizar_box-->";
			
			//Default settings; no box, and numbered list at bottom
			$div_box = "";
			$top = false;
			$bottom = true;
			
			//do we have configuration data?
			if  (count($indizar_config)>0)
			{
				$box = true;
				if($indizar_config['box_float'] == 'none') $box = false;
				
				$bottom = false;
				if($indizar_config['config'] & 2) $bottom = true;
				
				$top = false;
				if($indizar_config['config'] & 1) $top = true;
				
				if($box) {
					$float = $indizar_config['box_float'];
					$size = $indizar_config['box_size'];
					$div_box = "<div class='chapters' id='".$float."' style='width: ".$size."px;";
					if($float)
						$div_box.=" float: $float;";
					$div_box.="'>".$index."</div>";
				}
			}
			
			//if we define a box now we have an html code defined in the $div_box variable
			//if we don'use it the variable is empty.
			
			//if there isn't a box and we are in a chapter, use the box at the begin
			if(!strpos($content,$search_box)) {
				$content = str_replace ("<!--indizar_box_begin-->", $div_box, $content);
			}
			
			//use the box mark to place the box
			$content = str_replace ($search_box, $div_box, $content);
			
			//Do we use the numbered list at the begin?
			if($top)
				$content = "<p>$numbered_list</p> $content";
			
			//Do we use the numbered list at the end?
			if($bottom)
				$content .= " <p>$numbered_list</p>";		
		}
		
		//If we didn't use the box at the begin we have to delete it
		$content = str_replace ("<!--indizar_box_begin-->", "", $content);
	}
	
	return $content;
}

function index_list($list) {
	global $first_chapter, $chapter;

	$index="<ul>";
	$chap_index=$first_chapter;
	foreach($list as $item) {
		if($chapter!=$chap_index)
			$index.="<li><a href='".$item['url']."'>".$item['chapter']."</a></li>";
		else
			$index.="<li>".$item['chapter']."</li>";
		$chap_index++;
	}
	$index.="</ul>";
	return $index;
}

function numbered_list($list) { 

	global $wp_rewrite, $wp_query, $chapter, $chapters, $post, $page, $more, $multipage, $first_chapter,$indizar_config;

	$answer="<strong>".__("Chapters", 'indizar').":</strong> ";

	//If we aren't in the first chapter we declare the previous link
	if($chapter!=$first_chapter) {
		$answer.="| "."<a href='".$list[$chapter-1]['url']."'>".__("Previous", 'indizar' )."</a> | ";
	} else {
		$answer.="| ";
	}

	//We start the numerated list in the fisrts chapter
	$count=$first_chapter;

	foreach($list as $item) {
		$text = $count;
		
		//If we are in the chapter 0 it's the preface
		if($count == 0) {
			
			//The default value
			$text =  __("Preface", 'indizar');
			
			//The old school, use the user defined variable in the post if we have it
			if(strlen(get_post_meta($post->ID, 'indizar_null_chapter_title', true))>0) {
				$text = get_post_meta($post->ID, 'indizar_null_chapter_title', true);
			}
			
			//The new school, use the configuration if we have it
			if(strlen($indizar_config['preface_title'])>0) {
				$text = $indizar_config['preface_title'];
			}
		}
		
		//If the counter becomes the chapter were we are
		//just show the number
		//else create the link
		if($chapter == $count) {
			$answer.="$text | ";
		} else {
			$answer.="<a href='".$item['url']."'>$text</a> | ";
		}
		$count++;
	}

	//If we aren't at the last chapter, create the next link
	if($chapter!=$count-1) {
		$answer.="<a href='".$list[$chapter+1]['url']."'>".__("Next", 'indizar')."</a> |";
	}
	return $answer;

}

function chap_list() {
	global $chapters, $post, $first_chapter;

	$item=false;

	$page_index=$first_chapter;
	
	//if the prepost function poblated the chapters array 
	//create a new array and poblate it with titles and links to each chapter
	if($chapters) {
		$item=array();
		foreach($chapters as $chapter) {		
			$item[$page_index]['chapter']=$chapter;
			//The first chapter didn't use a number to define the chapter
			if ( $page_index == $first_chapter ) {
				$item[$page_index]['url']=get_permalink();
			} else {
				//Permalink without prety url
				if ( '' == get_option('permalink_structure') || in_array($post->post_status, array('draft', 'pending')) )
				$item[$page_index]['url']=get_permalink() . '&amp;chapter=' . $page_index;
				else //Permalink with prety url
				$item[$page_index]['url']=trailingslashit(get_permalink()) . user_trailingslashit("chapter/".$page_index, 'ch');
			}
			$page_index++;
		}
	}

	return $item;
}

function ind_addbuttons() {
	// Don't bother doing this stuff if the current user lacks permissions
	if ( !current_user_can('edit_posts') && !current_user_can('edit_pages') ) return;

	// Add only in Rich Editor mode
	if ( get_user_option('rich_editing') == 'true') {
	
		// add the button for wp21 in a new way
		add_filter('mce_external_plugins',  'add_ind_script');
		add_filter('mce_buttons',  'add_ind_button');
	}
}

function add_ind_button($buttons) {
	array_push($buttons, 'Indizar');
	return $buttons;
}

function add_ind_script($plugins) {
	$dir_name = '/wp-content/plugins/indizar';
	$url=get_bloginfo('wpurl');
	$pluginURL =  $url.$dir_name.'/tinymce/editor_plugin.js';
	$plugins['Indizar'] = $pluginURL;
	return $plugins;
}

?>
